# PerlTypes.pl generated by Makefile.PL.  Don't edit this file, edit Makefile.PL.
# From Convert::Binary::C $c->parse("#include <EXTERN.h>
#include <perl.h>")->configure

package C::DynaLib::PerlTypes;

$PerlTypes = {
               'Define' => [],
               'StdCVersion' => 199901,
               'ByteOrder' => 'LittleEndian',
               'LongSize' => 4,
               'IntSize' => 4,
               'HostedC' => 1,
               'ShortSize' => 2,
               'HasMacroVAARGS' => 1,
               'Assert' => [],
               'UnsignedChars' => 0,
               'DoubleSize' => 8,
               'CharSize' => 1,
               'EnumType' => 'Integer',
               'PointerSize' => 4,
               'EnumSize' => 4,
               'DisabledKeywords' => [],
               'FloatSize' => 4,
               'Alignment' => 1,
               'LongLongSize' => 8,
               'LongDoubleSize' => 12,
               'KeywordMap' => {},
               'Include' => [
                              '/usr/lib/perl5/5.10/i686-cygwin/CORE',
                              '/usr/include',
                              '',
                              '/usr/local/include /opt/local/include /usr/gnu/include /opt/gnu/include /usr/GNU/include /opt/GNU/include'
                            ],
               'HasCPPComments' => 1,
               'Bitfields' => {
                                'Engine' => 'Generic'
                              },
               'UnsignedBitfields' => 0,
               'Warnings' => 0,
               'CompoundAlignment' => 1,
               'OrderMembers' => 0
             };
;

1;
__END__
